from django.conf import settings
from django.core.exceptions import ValidationError
from django.template.defaultfilters import filesizeformat
from django.utils.encoding import force_str
from django.utils.html import format_html
from django.utils.safestring import mark_safe
import logging
import warnings

from . import parser
from attachments.models import Attachment

logger = logging.getLogger(__name__)


class BBCode:
    valid_packs = parser.get_valid_bbcode_packs()

    ATTACH_TEMPLATE = """
    <div class="inline_attachment">
        <img src="{0}bbcode/file_extension_{1}.png" />
        <div><a href="{2}">{3}</a></div>
        Taille : {4}
    </div>
    """.strip()

    IMAGE_TEMPLATE = """
        <a href="{0}"><img src="{1}" alt="" /></a>
    """.strip()

    def __init__(self, pack, text, from_db):
        assert pack is not None

        self.can_have_attachments = pack.endswith("_attach")
        self._validated = False

        if from_db:
            self._node = parser.deserialize(pack, text)
            self._validated = True
        else:
            try:
                self._node = parser.parse_user_input(pack, text)
            except parser.BBCodeParseError as e:
                raise ValidationError(e)

    def as_bbcode(self):
        return self._node.as_bbcode()

    def as_text(self):
        return self._node.as_text()

    def as_html(self):
        items = self._node.get_attachments()
        attachments = Attachment.objects.in_bulk(items)

        def get_attachment_html(ident):
            value = attachments.get(ident)

            if value is None:
                logger.error("Pièce jointe {} non trouvée", ident)
                return ""

            if value.ident_mini:
                return format_html(self.IMAGE_TEMPLATE, value.get_absolute_url(), value.get_mini_url())

            # XXX filesizeformat passe un float à ngettext, qui génère un
            # warning (corrigé dans Django 3.0a1)
            with warnings.catch_warnings():
                warnings.simplefilter("ignore", DeprecationWarning)
                formatted_size = filesizeformat(value.file_size)

            return format_html(
                self.ATTACH_TEMPLATE,
                settings.STATIC_URL,
                value.file_type,
                value.get_absolute_url(),
                value.file_name,
                formatted_size,
            )

        html = self._node.as_html(attachment_callback=get_attachment_html)
        return mark_safe(html)

    def validate_attachments(self, namespace, owner_id):
        if not self.can_have_attachments:
            raise Exception("Attachements are not supported with this " "BBCode type.")

        if owner_id is None:
            owner_id = 0

        items = self._node.get_attachments()

        Attachment.objects.update_attachments_for(namespace, owner_id, items, commit=False)

        self._validated = True

    def commit_attachments(self, namespace, owner_id):
        if not self.can_have_attachments:
            raise Exception("Attachements are not supported with this " "BBCode type.")

        assert owner_id

        items = self._node.get_attachments()

        Attachment.objects.update_attachments_for(namespace, owner_id, items, commit=True)

    def disown_attachments(self, namespace, owner_id):
        if not self.can_have_attachments:
            raise Exception("Attachements are not supported with this " "BBCode type.")

        assert owner_id

        Attachment.objects.update_attachments_for(namespace, owner_id, [], commit=True)

    def get_main_attachment(self):
        if not self.can_have_attachments:
            raise Exception("Attachements are not supported with this " "BBCode type.")

        attachment_ids = self._node.get_attachments()
        attachments_dict = Attachment.objects.in_bulk(attachment_ids)

        for attachment_id in attachment_ids:
            attachment = attachments_dict.get(attachment_id, None)
            if attachment is not None and attachment.ident_mini:
                return attachment

    def serialize(self):
        if self.can_have_attachments and not self._validated:
            raise RuntimeError(
                "Ce BBCode utilise des pièces jointes mais "
                "n’a pas été validé. Appelez validate_attachments (ou "
                "utilisez un BBCodeModelForm)."
            )

        return self._node.serialize()

    def summary(self, soft_limit=100, hard_limit=400):
        items = self._node.as_text().split("\n")
        text = ""
        count = 0
        for item in items:
            if not item:
                continue

            new_count = count + len(item)
            if new_count > hard_limit:
                text += item[0 : hard_limit - count] + "…"
                return text

            text += ("\n" if text else "") + item
            if new_count > soft_limit:
                return text

            count = new_count

        return text

    def __str__(self):
        return force_str(self._node.as_bbcode())
